"use strict";

var _extends2 = require("babel-runtime/helpers/extends");

var _extends3 = _interopRequireDefault(_extends2);

var _promise = require("babel-runtime/core-js/promise");

var _promise2 = _interopRequireDefault(_promise);

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

var _LoginLogic = require("./LoginLogic");

var _LoginLogic2 = _interopRequireDefault(_LoginLogic);

var _sinon = require("sinon");

var _sinon2 = _interopRequireDefault(_sinon);

var _chai = require("chai");

var _enzymeSetup = require("../utilities/enzyme-setup");

var _jsonwebtoken = require("jsonwebtoken");

var _jsonwebtoken2 = _interopRequireDefault(_jsonwebtoken);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var voidFn = function voidFn() {};
var voidPromiseFn = function voidPromiseFn() {
    return new _promise2.default(voidFn);
};

var expiredToken = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJkYXRhIjp7InVzZXJJZCI6Mn0sImlhdCI6MTUzNTEzOTE1OSwiZXhwIjoxNTM1MTQyNzU5fQ.gcu47VrcvYSaIdqkW0D1rPbwW-Okl9Wm97JEbDY99-M";
var freshToken = function freshToken(userId) {
    var token = _jsonwebtoken2.default.sign({
        data: { userId: userId },
        exp: Math.floor(new Date().getTime() / 1000) + 7 * 24 * 60 * 60 // Note: in seconds!
    }, "123"); // secret is defined in the environment variable JWT_SECRET
    return token;
};

var mock = function mock(props, children) {
    return (0, _enzymeSetup.shallow)(_react2.default.createElement(
        _LoginLogic2.default,
        (0, _extends3.default)({
            onRequireLogin: voidFn,
            fetchRefreshToken: voidPromiseFn,
            getCookie: voidFn,
            fetchMe: voidFn
        }, props),
        children || voidFn
    ));
};

describe("LoginLogic", function () {

    describe("when not logged in", function () {

        it("checks the cookies for an auth-token", function () {
            var getCookie = _sinon2.default.spy();
            // mock({getCookie={getCookie} />)
            mock({ getCookie: getCookie });
            (0, _chai.expect)(getCookie).to.have.property("callCount", 1);
        });

        it("checks the URL hash for an auth-token", function () {
            var getLocationHash = _sinon2.default.spy(function () {
                return "";
            });
            mock({ getLocationHash: getLocationHash });
            (0, _chai.expect)(getLocationHash).to.have.property("callCount", 1);
        });

        it("calls onRequireLogin callback when no token is present", function () {
            var onRequireLogin = _sinon2.default.spy();
            mock({ onRequireLogin: onRequireLogin });
            (0, _chai.expect)(onRequireLogin).to.have.property("callCount", 1);
        });

        it("calls onRequireLogin callback when an expired token is present", function () {
            var getLocationHash = function getLocationHash() {
                return "#token=" + expiredToken;
            };
            var onRequireLogin = _sinon2.default.spy();
            mock({ getLocationHash: getLocationHash, onRequireLogin: onRequireLogin });
            (0, _chai.expect)(onRequireLogin).to.have.property("callCount", 1);
        });
    });

    describe("when a valid token is found", function () {

        it("calls fetchMe()", function () {
            var fetchMeNot = function fetchMeNot() {
                return done("FetchMe should not have been called without token");
            };
            var authToken = freshToken(2);
            var getEmptyHash = function getEmptyHash() {
                return "#";
            };
            var getLocationHash = function getLocationHash() {
                return "#token=" + authToken;
            };

            mock({ fetchMe: fetchMeNot, getLocationHash: getEmptyHash });

            return new _promise2.default(function (resolve, reject) {
                var fetchMe = function fetchMe(token) {
                    try {
                        (0, _chai.expect)(token).to.equal(authToken);
                        resolve();
                    } catch (err) {
                        reject(err);
                    }
                };
                mock({ fetchMe: fetchMe, getLocationHash: getLocationHash });
            });
        });

        it("calls onReceiveToken()", function () {
            // const onReceiveToken = sinon.spy()
            var authToken = freshToken(2);
            var getLocationHash = function getLocationHash() {
                return "#token=" + authToken;
            };
            window.location.hash = "#token=" + authToken;

            return new _promise2.default(function (resolve, reject) {
                var onReceiveToken = function onReceiveToken(token) {
                    try {
                        (0, _chai.expect)(token).to.equal(authToken);
                        resolve();
                    } catch (err) {
                        reject(err);
                    }
                };
                mock({ onReceiveToken: onReceiveToken, getLocationHash: getLocationHash });
            });
        });
    });

    describe("when fetchMe() succeeds", function () {

        it("calls fetchRefresToken()", function () {
            var authToken = freshToken(2);
            var getLocationHash = function getLocationHash() {
                return "#token=" + authToken;
            };
            var fetchMe = function fetchMe() {
                return new _promise2.default(function (r) {
                    return r({ id: 3 });
                });
            };

            return new _promise2.default(function (resolve, reject) {
                var fetchRefreshToken = function fetchRefreshToken(token) {
                    try {
                        (0, _chai.expect)(token).to.equal(authToken);
                        resolve();
                    } catch (err) {
                        reject(err);
                    }
                };
                mock({ fetchMe: fetchMe, getLocationHash: getLocationHash, fetchRefreshToken: fetchRefreshToken });
            });
        });
    });

    describe("when logged in", function () {
        var userObject = { id: 1 };
        var authToken = freshToken(2);
        var fetchMe = function fetchMe() {
            return new _promise2.default(function (resolve) {
                return resolve(userObject);
            });
        };

        it("injects the user object", function (done) {
            var innerFn = function innerFn(_ref) {
                var auth = _ref.auth;

                if (auth.status !== 'loggedIn') return;
                (0, _chai.expect)(auth).to.have.property('currentUser', userObject);
                done();
            };
            mock({ fetchMe: fetchMe }, innerFn);
        });

        it("shows the loggedIn status", function (done) {
            var innerFn = function innerFn(_ref2) {
                var auth = _ref2.auth;

                if (auth.status === 'loggedIn') done();
            };
            mock({ fetchMe: fetchMe }, innerFn);
        });
    });

    describe("when logging out", function () {

        it("calls onLogout", function () {
            var authToken = freshToken(2);
            var getLocationHash = function getLocationHash() {
                return "#token=" + authToken;
            };
            var fetchMe = function fetchMe() {
                return new _promise2.default(function (r) {
                    return r({ id: 3 });
                });
            };
            var fetchRefreshToken = function fetchRefreshToken() {
                return new _promise2.default(function (r) {
                    return r(123);
                });
            };

            return new _promise2.default(function (resolve, reject) {
                var onLogout = function onLogout() {
                    resolve();
                };
                var trigger = function trigger(_ref3) {
                    var auth = _ref3.auth;

                    auth.logout();
                    return "";
                };
                mock({ fetchMe: fetchMe, getLocationHash: getLocationHash, fetchRefreshToken: fetchRefreshToken, onLogout: onLogout }, trigger);
            });
        });
    });
});