'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _toArray2 = require('babel-runtime/helpers/toArray');

var _toArray3 = _interopRequireDefault(_toArray2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _memoize = require('../utilities/memoize');

var _debounce = require('../utilities/debounce');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var recSort = function recSort(a, b, _ref) {
    var _ref2 = (0, _toArray3.default)(_ref),
        key = _ref2[0],
        tail = _ref2.slice(1);

    return (key.asc ? 1 : -1) * ( // when not ascending, flip
    a[key.id] > b[key.id] ? 1 : a[key.id] < b[key.id] ? -1 : tail.length > 0 ? recSort(a, b, tail) : 0);
};

var basicSort = function basicSort(data, ordering) {
    return data.slice(0).sort(function (a, b) {
        return ordering.length > 0 ? recSort(a, b, ordering) : 0;
    });
};

var basicFilter = function basicFilter(data, searchString) {
    return data.filter(function (row) {
        if (searchString === '') return true;
        for (var key in row) {
            if (row.hasOwnProperty(key)) {
                if (typeof row[key] === 'string' && row[key].toLowerCase().match(searchString)) return true;
            }
        }
        return false;
    });
};

var updatesOnly = function updatesOnly(original, changes) {
    return _lodash2.default.fromPairs(_lodash2.default.flatMap(changes, function (newValue, key) {
        return newValue === original[key] ? [] : [[key, newValue]];
    }));
};

var paginate = function paginate(data, page, pageLimit) {
    return data.slice(page * pageLimit, (page + 1) * pageLimit);
};

/**
 * Create a memoized recomputation function, which will only apply
 * the filter, sorting and pagination if values or functions have
 * changed.
 */
var makeComputeViewData = function makeComputeViewData() {
    var $filter = (0, _memoize.memoize)(function (data, filterFn, query) {
        return filterFn(data, query);
    });
    var $sort = (0, _memoize.memoize)(function (data, sortFn, ordering) {
        return sortFn(data, ordering);
    });

    return function (data, filterFn, query, sortFn, ordering) {
        return $sort($filter(data, filterFn, query), sortFn, ordering);
    };
};

var RecordTableLogic = function (_Component) {
    (0, _inherits3.default)(RecordTableLogic, _Component);

    function RecordTableLogic(props) {
        (0, _classCallCheck3.default)(this, RecordTableLogic);

        // This binding is necessary to make `this` work in the callback
        var _this = (0, _possibleConstructorReturn3.default)(this, (RecordTableLogic.__proto__ || (0, _getPrototypeOf2.default)(RecordTableLogic)).call(this, props));

        _this.state = {
            search: '',
            ordering: [],
            page: 0,
            pageLimit: 10,
            writingToUrl: false
        };
        _this.writeToUrl = (0, _debounce.debounce)(_this.writeToUrl.bind(_this), 3000);
        _this.handleSearchChange = _this.handleSearchChange.bind(_this);
        _this.handleOrderingChange = _this.handleOrderingChange.bind(_this);
        _this.handlePageChange = _this.handlePageChange.bind(_this);
        _this.handlePageLimitChange = _this.handlePageLimitChange.bind(_this);
        _this.computeViewData = makeComputeViewData();
        return _this;
    }

    (0, _createClass3.default)(RecordTableLogic, [{
        key: 'componentWillReceiveProps',
        value: function componentWillReceiveProps(props) {
            // Do not change state to URL when its synchronizing
            if (this.state.writingToUrl) return this.setState({ writingToUrl: false });

            this.readFromUrl(props.navigation);
        }
    }, {
        key: 'writeToUrl',
        value: function writeToUrl() {
            var _this2 = this;

            this.setState({
                writingToUrl: true
            }, function () {
                _this2.props.navigation.setParams({
                    search: _this2.state.search,
                    ordering: _this2.state.ordering,
                    page: _this2.state.page,
                    pageLimit: _this2.state.pageLimit
                });
            });
        }
    }, {
        key: 'getViewData',
        value: function getViewData() {
            return this.computeViewData(this.props.data, this.props.filter, this.state.search, this.props.sort, this.state.ordering);
        }
    }, {
        key: 'readFromUrl',
        value: function readFromUrl(navigation) {
            var _navigation = navigation || this.props.navigation;
            var params = ['search', 'ordering', 'page', 'pageLimit'];
            this.setState(updatesOnly(_lodash2.default.pick(this.state, params), _lodash2.default.pick(_navigation, params)));
        }
    }, {
        key: 'handleSearchChange',
        value: function handleSearchChange(value) {
            this.setState({
                search: value
            }, this.writeToUrl());
        }
    }, {
        key: 'handleOrderingChange',
        value: function handleOrderingChange(value) {
            this.setState({
                ordering: value
            }, this.writeToUrl());
        }
    }, {
        key: 'handlePageChange',
        value: function handlePageChange(value) {
            this.setState({
                page: value
            }, this.writeToUrl());
        }
    }, {
        key: 'handlePageLimitChange',
        value: function handlePageLimitChange(value) {
            this.setState({
                pageLimit: value
            }, this.writeToUrl());
        }
    }, {
        key: 'render',
        value: function render() {
            var _state = this.state,
                ordering = _state.ordering,
                search = _state.search,
                page = _state.page,
                pageLimit = _state.pageLimit;
            var _props = this.props,
                Table = _props.Table,
                tableProps = _props.tableProps;


            var data = this.getViewData();

            var table = {
                search: search,
                data: data,
                viewData: paginate(data, page, pageLimit),
                page: page,
                pageLimit: pageLimit,
                ordering: ordering,
                onSearchChange: this.handleSearchChange,
                onOrderingChange: this.handleOrderingChange,
                onPageChange: this.handlePageChange,
                onPageLimitChange: this.handlePageLimitChange
            };

            return _react2.default.createElement(Table, (0, _extends3.default)({ table: table }, tableProps));
        }
    }]);
    return RecordTableLogic;
}(_react.Component);

RecordTableLogic.propTypes = {
    Table: _propTypes2.default.func.isRequired,
    data: _propTypes2.default.array.isRequired,
    tableProps: _propTypes2.default.object
};

RecordTableLogic.defaultProps = {
    /**
     * Sort-function
     * (data, ordering) => orderedData
     */
    sort: basicSort,
    filter: basicFilter
};

exports.default = RecordTableLogic;